/*
 *  Copyright (C) 2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
 *  Note   : based on a sdl-rendering example from Mirco "MacSlow" Mueller <macslow@bangang.de>
*/

#include <stdio.h>

#include "SDL.h"
#include "SDL_opengl.h"
#include "sdl-helper.h"

/* now this here is _really_ nasty, but I'm too lazy to do it right */
#define GL_TEXTURE_RECTANGLE_ARB 0x84F5

#define GL_I_CHANNELS 4

static unsigned int   puiTextureId = 0;
static SDL_Surface   *pSurfaceWindow = NULL;

void kiba_sdl_init (unsigned long int window,
		    int               iWidth,
		    int               iHeight) {
	/* Hack to get SDL to use GTK window */
	{ char SDL_windowhack[32];
		sprintf(SDL_windowhack,"SDL_WINDOWID=%ld", window);
		putenv(SDL_windowhack);
	}
	
	/* Initialize SDL */
	if ((SDL_Init (SDL_INIT_VIDEO) == -1)) {
		fprintf (stderr, "SDL - Could not initialize SDL: %s.\n", SDL_GetError ());
		return;
	}
	
	kiba_sdl_update_surface (iWidth, iHeight);
	
	SDL_GL_SetAttribute (SDL_GL_RED_SIZE, 5);
	SDL_GL_SetAttribute (SDL_GL_GREEN_SIZE, 5);
	SDL_GL_SetAttribute (SDL_GL_BLUE_SIZE, 5);
	SDL_GL_SetAttribute (SDL_GL_DEPTH_SIZE, 16);
	SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, 1);
	
	glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
	glDisable (GL_DEPTH_TEST);
	/* Enable Alpha Blending (disable alpha testing) */
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	/* Enable Blending (disable alpha testing) */
	glEnable(GL_BLEND);
	glEnable (GL_TEXTURE_RECTANGLE_ARB);
}

void kiba_sdl_update_surface (int iWidth,
			      int iHeight) {
	if (pSurfaceWindow) {
		SDL_FreeSurface(pSurfaceWindow);
		pSurfaceWindow = NULL;
	}
	
	pSurfaceWindow = SDL_SetVideoMode (iWidth, iHeight,
		16, SDL_OPENGL | SDL_HWSURFACE | SDL_NOFRAME | SDL_RESIZABLE);
	if (!pSurfaceWindow)
		fprintf (stderr, "SDL - Couldn't open Window: %s\n", SDL_GetError ());
}

int kiba_sdl_draw_func (unsigned char **pucSurfaceData,
			int             iWidth,
			int             iHeight) {
	if (!*pucSurfaceData) {
		fprintf (stderr, "SDL draw func - No valid pointer to surface-data\n");
		return 0;
	}

	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();
	/* Clear the color buffers. */
	glClear (GL_COLOR_BUFFER_BIT);

	glPushMatrix ();
	
	glBindTexture (GL_TEXTURE_RECTANGLE_ARB, puiTextureId);
	glTexImage2D (GL_TEXTURE_RECTANGLE_ARB,
				  0,
				  GL_RGBA,
				  iWidth,
				  iHeight,
				  0,
				  GL_BGRA,
				  GL_UNSIGNED_BYTE,
				  *pucSurfaceData);
	
	glBegin (GL_QUADS);
		glTexCoord2f (0.0f, (GLfloat) iHeight);
		glVertex2f (0.0f, 0.0f);
		glTexCoord2f ((GLfloat) iWidth, (GLfloat) iHeight);
		glVertex2f (1.0f, 0.0f);
		glTexCoord2f ((GLfloat) iWidth, 0.0);
		glVertex2f (1.0f, 1.0f);
		glTexCoord2f (0.0f, 0.0);
		glVertex2f (0.0f, 1.0f);
	glEnd ();
	
	glPopMatrix ();
	
	SDL_GL_SwapBuffers();
	
	return 1;
}

void kiba_sdl_resize_func (int iWidth,
			   int iHeight) {
	glViewport (0, 0, iWidth, iHeight);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glOrtho (0.0f, 1.0f, 0.0f, 1.0f, -1.0f, 1.0f);

	glClear (GL_COLOR_BUFFER_BIT);

	glDeleteTextures (1, &puiTextureId);
	glGenTextures (1, &puiTextureId);
	glBindTexture (GL_TEXTURE_RECTANGLE_ARB, puiTextureId);
	glTexImage2D (GL_TEXTURE_RECTANGLE_ARB,
				  0,
				  GL_RGBA,
				  iWidth,
				  iHeight,
				  0,
				  GL_BGRA,
				  GL_UNSIGNED_BYTE,
				  NULL);
	
	/* make the window transparent */
	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);	
}

void kiba_sdl_fini () {
	glDeleteTextures (1, &puiTextureId);
	if (pSurfaceWindow)
		SDL_FreeSurface(pSurfaceWindow);
	SDL_Quit ();
}

